package com.huiju.estateagents.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.bo.request.QrCodeStatisticResponseBO;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.WxQrCodeUtils;
import com.huiju.estateagents.entity.TaMiniapp;
import com.huiju.estateagents.entity.TaQrcode;
import com.huiju.estateagents.mapper.TaMiniappMapper;
import com.huiju.estateagents.mapper.TaQrcodeMapper;
import com.huiju.estateagents.service.IMiniAppService;
import com.huiju.estateagents.service.ITaQrcodeService;
import org.checkerframework.checker.units.qual.A;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * <p>
 * 二维码管理表 服务实现类
 * </p>
 *
 * @author fxf
 * @since 2020-05-20
 */
@Service
public class TaQrcodeServiceImpl extends ServiceImpl<TaQrcodeMapper, TaQrcode> implements ITaQrcodeService {

    private Logger logger = LoggerFactory.getLogger(TaQrcodeServiceImpl.class);

    @Autowired
    private TaQrcodeMapper taQrcodeMapper;

    @Autowired
    private TaMiniappMapper taMiniappMapper;

    @Autowired
    private IMiniAppService iMiniAppService;

    /**
     * 条件查询二维码列表
     *
     * @param pageNum
     * @param pageSize
     * @param targetType
     * @param targetName
     * @param channelId
     * @param buildingId
     * @param orgId
     * @return
     */
    @Override
    public ResponseBean listQrCodeByCondition(Integer pageNum, Integer pageSize, String targetType, String targetName, Integer channelId, String buildingId, Integer orgId) {
        ResponseBean responseBean = new ResponseBean();
        IPage<TaQrcode> pg = new Page<>(pageNum, pageSize);
        responseBean.addSuccess(taQrcodeMapper.listQrCodeByCondition(pg, targetType, targetName, channelId, buildingId, orgId));
        return responseBean;
    }

    /**
     * 保存二维码信息
     *
     * @param taQrcode
     * @param userId
     * @param orgId
     * @return
     */
    @Override
    public ResponseBean saveQrcode(TaQrcode taQrcode, Integer userId, Integer orgId) {
        ResponseBean responseBean = new ResponseBean();

        // 保存数据
        taQrcode.setOrgId(orgId);
        taQrcode.setCreateUser(userId);
        taQrcode.setCreateDate(LocalDateTime.now());
        taQrcode.setStatus(CommConstant.STATUS_NORMAL);
        if (!save(taQrcode)) {
            responseBean.addError("fail");
            return responseBean;
        }

        // 获取appId
        QueryWrapper<TaMiniapp> taMiniappQuery = new QueryWrapper<>();
        taMiniappQuery.eq("org_id", orgId);
        TaMiniapp taMiniapp = taMiniappMapper.selectOne(taMiniappQuery);

        // 生成二维码
        String paramsStr = WxQrCodeUtils.generateSceneAndPage(taQrcode.getTargetType(), "",
                taQrcode.getTargetId(), taQrcode.getTargetType(), taQrcode.getChannelId() + "", taQrcode.getQrCodeId() + "", taQrcode.getBuildingId());
        ResponseBean qrCodeResult = iMiniAppService.getQrCode(paramsStr, taMiniapp.getMiniappId());
        if (qrCodeResult != null && ResponseBean.CODE_SUCCESS == qrCodeResult.getCode()) {
            // 更新数据
            taQrcode.setQrCodeUrl(qrCodeResult.getCode() + "");
            if (updateById(taQrcode)) {
                responseBean.addSuccess(taQrcode);
                return responseBean;
            }
        }
        removeById(taQrcode.getQrCodeId());
        responseBean.addError("fail");
        return responseBean;
    }

    /**
     * 获取二维码统计
     *
     * @param qrCodeId
     * @param orgId
     * @return
     */
    @Override
    public ResponseBean getStatisticData(Integer qrCodeId, Integer orgId) {
        ResponseBean responseBean = new ResponseBean();

        // 获取二维码信息
        TaQrcode taQrcode = taQrcodeMapper.selectById(qrCodeId);
        if (taQrcode == null) {
            responseBean.addError("fail");
            return responseBean;
        }

        // 获取统计数据
        QrCodeStatisticResponseBO statisticData = taQrcodeMapper.getStatisticData(orgId);

        // 组装反参
        JSONObject resultObj = new JSONObject();
        resultObj.put("statistic", statisticData);
        resultObj.put("qrCodeInfo", taQrcode);
        responseBean.addSuccess(resultObj);
        return responseBean;
    }
}
